'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _atomdoc = require('atomdoc');

var _atomdoc2 = _interopRequireDefault(_atomdoc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 *  CommentParser Class
 */
var CommentParser = function () {
  /**
   *  Public: creates a new CommentParser instance
   *
   *  * `commentsArray` (optional) {Array} an array to add to the instance as the
   *    `comments` property.
   *
   *  ## Examples
   *
   *  Designed to be used by `falafel` to parse javascript comments.
   *
   *  ```js
   *  const commentParser = new CommentParser();
   *  falafel(this.content, {
   *    sourceType: 'module',
   *    ecmaVersion: '6',
   *    onComment: commentParser.parseComment.bind(commentParser),
   *    locations: true,
   *    allowHashBang: true,
   *  }, (node) => {
   *    contentInspector.inspectNode(node);
   *    contentParser.parseNode(node);
   *  });
   *  ```
   */
  function CommentParser() {
    var commentsArray = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    _classCallCheck(this, CommentParser);

    this.comments = commentsArray;
  }
  /**
   *  Private: checks if comment is a valid AtomDoc block comment.
   *
   *  * `block` {Boolean} true if comment is a block comment.
   *  * `text` {String} block comment value.
   *
   *  Returns {Boolean} true if AtomDoc.
   */


  _createClass(CommentParser, [{
    key: '_isAtomDocComment',
    value: function _isAtomDocComment(block, text) {
      if (!block) return false;
      return Boolean(text.match(/^[*\s]*(Private:|Public:|Internal:)/));
    }
    /**
     *  Private: adjusts contents of block comments so the base indent is 0.
     *
     *  * `str` {String} block comment.
     *
     *  Returns {String} adjusted content.
     */

  }, {
    key: '_fixIndent',
    value: function _fixIndent(str) {
      var indentRegEx = /^(\s)*/;
      var indent = str.match(indentRegEx)[0];
      return str.replace(new RegExp(indent, ['g']), '\n').trim();
    }

    /**
     *  Private: removes astricks from block comments content.
     *
     *  * `str` {String} block comment value.
     *
     *  Returns {String} of block comment without value.
     */

  }, {
    key: '_removeBlockAstrisks',
    value: function _removeBlockAstrisks(str) {
      if (str.match(/^\*/)) {
        return str.replace(/^\s*\*/gm, '');
      }
      return str;
    }

    /**
     *  Public: a function that handles comments. Adds result to _comments {Array}.
     *
     *  * `block` {Boolean} true if comment is a block comment.
     *  * `text` {String} block comment value.
     *  * `start` {Integer} the start of the comment.
     *  * `end` {Integer} the end of the comment.
     *
     *  ## Examples
     *
     *  Designed to be used by `falafel` to parse javascript comments.
     *
     *  ```js
     *  const commentParser = new CommentParser();
     *  falafel(this.content, {
     *    sourceType: 'module',
     *    ecmaVersion: '6',
     *    onComment: commentParser.parseComment.bind(commentParser),
     *    locations: true,
     *    allowHashBang: true,
     *  }, (node) => {
     *    contentInspector.inspectNode(node);
     *    contentParser.parseNode(node);
     *  });
     *  ```
     */

  }, {
    key: 'parseComment',
    value: function parseComment(block, text, start, end) {
      if (this._isAtomDocComment(block, text)) {
        var value = this._fixIndent(this._removeBlockAstrisks(text));
        var atomDoc = _atomdoc2.default.parse(value);
        this.comments.push(Object.assign(atomDoc, { start: start, end: end }));
      }
    }
  }]);

  return CommentParser;
}();

exports.default = CommentParser;
module.exports = exports['default'];