'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 *  InspectorMethod Class
 */
var InspectorMethod =
/**
 *  Private: a typed object to hold the expected results of inspecting the javascript content.
 *
 *  * `type` {String} function type.
 *  * `name` {String} the identifier name for the function.
 *  * `className` {String} of the class that contains the method, or {false} if
 *    the method is not part of a class.
 *  * `start` {Int} the character start point of the method.
 *  * `end` {Int} the character end point of the method.
 *  * `definitionLine` {Int} the line number of the start of the definition of the method.
 *  * `args` {Array} of {Param}
 *  * `returns` {Array} of {ReturnResult}
 */
function InspectorMethod(type, name, className, start, end, definitionLine, args, returns) {
  _classCallCheck(this, InspectorMethod);

  var params = { type: type, name: name, className: className, start: start, end: end, definitionLine: definitionLine, args: args, returns: returns };
  Object.assign(this, params);
};

var privateReturnProps = new WeakMap();

/**
 *  ReturnResult Class
 */

var ReturnResult = function () {
  /**
   *  Private: a typed object to hold the expected results of the returns contained
   *  in the method.
   *
   *  * `start` {Int} the character start point of the return statement.
   *  * `end` {Int} the character end point of the return statement.
   *  * `funcLine` {Int} definition line number of function ended by return.
   *  * `name` (optional) {String} identifier in the return statement if it has one, else is false.
   */
  function ReturnResult(start, end, funcLine) {
    var name = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

    _classCallCheck(this, ReturnResult);

    var params = { start: start, end: end, name: name };
    Object.assign(this, params);
    privateReturnProps.set(this, { funcLine: funcLine });
  }
  /**
   *  Public: getter for func property so it doesn't show up when comparing the object in the test.
   *
   *  ## Examples
   *
   *  ```js
   *  const returnResult = new ReturnResult(1,2,node.source);
   *  returnResult.funcLine; // node.source
   *  ```
   *
   *  Returns {Int} definition line number of function ended by return.
   */


  _createClass(ReturnResult, [{
    key: 'funcLine',
    get: function get() {
      return privateReturnProps.get(this).funcLine;
    }
  }]);

  return ReturnResult;
}();

/**
 *  Param Class
 */


var Param =
/**
 *  Private: a typed object to hold the expected results of the parameters found
 *  in the method definition.
 *
 *  * `name` {String} parameter identifier.
 *  * `optional` (optional) {Boolean} if the parameter has a default value this
 *    is true, otherwise false.
 */
function Param(name) {
  var optional = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  _classCallCheck(this, Param);

  this.name = name;
  this.optional = optional;
};

/**
 *  Private: finds `ReturnStatement` {Node}s within a function.
 *
 *  * `node` {Node} Spidermonkey AST node.
 *  * `returnsArr` {Array} of {ReturnResult}s from parsing all the `ReturnStatement` {Node}s.
 *
 *  Returns {Array} of {ReturnResult}s that have definitions that are contained within
 *  the `node` `start` and `end`.
 */


function _getReturns(node, returnsArr) {
  return returnsArr.filter(function (returnResult) {
    return Boolean(node.loc.start.line === returnResult.funcLine);
  });
}

/**
 *  Private: finds parameters.
 *
 *  * `params` {Array} of Identifier {Node}s describing parameters.
 *
 *  Returns {Array} of {Param}.
 */
function _parseParams(params) {
  return params.map(function (param) {
    var result = void 0;
    switch (param.type) {
      case 'Identifier':
        result = new Param(param.name);
        break;
      case 'AssignmentPattern':
        result = new Param(param.left.name, true);
        break;
      default:
    }
    return result;
  });
}

/**
 *  Private: uses `nodeType` to determine which handler method should be used to
 *  parse a node.
 *
 *  * `nodeType` {String} the `type` property of a {Node}.
 *  * `handlerObj` {Object} of {Function}s whose property names match {Node} `type`.
 *
 *  Returns {Function} when the `nodeType` matches a `handlerObj` property.
 *  Returns {false} when no `nodeType` matches.
 */
function _getHandler(nodeType, handlerObj) {
  if ({}.hasOwnProperty.call(handlerObj, nodeType)) {
    return handlerObj[nodeType];
  }
  return false;
}

var _typeHandlers = {
  /**
   *  Private: parses `ArrowFunctionExpression` {Node}s
   *
   *  * `node` {Node} of type `ArrowFunctionExpression`
   *  * `arr` {Array} of {InspectorMethod} results. Results of parsing this {Node}
   *    will be added to this.
   *  * `returnsArr` {Array} of {ReturnResult}.
   */
  ArrowFunctionExpression: function ArrowFunctionExpression(node, arr, returnsArr) {
    if (node.parent.type === 'VariableDeclarator' || node.parent.type === 'ExpressionStatement') {
      var name = false;
      try {
        name = node.parent.id.name;
      } catch (err) {/* */}
      var args = _parseParams(node.params);
      var returns = _getReturns(node, returnsArr);
      var result = new InspectorMethod(node.type, name, false, node.start, node.end, node.loc.start.line, args, returns);
      arr.push(result);
    }
    if (node.parent.type === 'AssignmentExpression') {
      var _name = node.parent.left.name;
      var _args = _parseParams(node.params);
      var _returns = _getReturns(node, returnsArr);
      var _result = new InspectorMethod(node.type, _name, false, node.start, node.end, node.loc.start.line, _args, _returns);
      arr.push(_result);
    }
  },

  /**
   *  Private: parses `FunctionDeclaration` {Node}s
   *
   *  * `node` {Node} of type `FunctionDeclaration`
   *  * `arr` {Array} of {InspectorMethod} results. Results of parsing this {Node}
   *    will be added to this.
   *  * `returnsArr` {Array} of {ReturnResult}.
   */
  FunctionDeclaration: function FunctionDeclaration(node, arr, returnsArr) {
    var name = node.id.name || false;
    var args = _parseParams(node.params);
    var returns = _getReturns(node, returnsArr);
    var result = new InspectorMethod(node.type, name, false, node.start, node.end, node.loc.start.line, args, returns);
    arr.push(result);
  },

  /**
   *  Private: parses `FunctionExpression` {Node}s
   *
   *  * `node` {Node} of type `FunctionExpression`
   *  * `arr` {Array} of {InspectorMethod} results. Results of parsing this {Node}
   *    will be added to this.
   *  * `returnsArr` {Array} of {ReturnResult}.
   */
  FunctionExpression: function FunctionExpression(node, arr, returnsArr) {
    if (node.parent.type === 'Property') {
      var name = node.parent.key.name || false;
      var args = _parseParams(node.params);
      var returns = _getReturns(node, returnsArr);
      var result = new InspectorMethod(node.type, name, false, node.start, node.end, node.loc.start.line, args, returns);
      arr.push(result);
    }
    if (node.parent.type === 'VariableDeclarator') {
      var _name2 = node.parent.id.name || false;
      var _args2 = _parseParams(node.params);
      var _returns2 = _getReturns(node, returnsArr);
      var _result2 = new InspectorMethod(node.type, _name2, false, node.start, node.end, node.loc.start.line, _args2, _returns2);
      arr.push(_result2);
    }
    if (node.parent.type === 'AssignmentExpression') {
      var _name3 = node.parent.left.name;
      var _args3 = _parseParams(node.params);
      var _returns3 = _getReturns(node, returnsArr);
      var _result3 = new InspectorMethod(node.type, _name3, false, node.start, node.end, node.loc.start.line, _args3, _returns3);
      arr.push(_result3);
    }
  },

  /**
   *  Private: parses `MethodDefinition` {Node}s
   *
   *  * `node` {Node} of type `MethodDefinition`
   *  * `arr` {Array} of {InspectorMethod} results. Results of parsing this {Node}
   *    will be added to this.
   *  * `returnsArr` {Array} of {ReturnResult}.
   */
  MethodDefinition: function MethodDefinition(node, arr, returnsArr) {
    var name = node.key.name;
    var args = _parseParams(node.value.params);
    var className = false;
    try {
      className = node.parent.parent.id.name;
    } catch (err) {/**/}
    var returns = _getReturns(node, returnsArr);
    var result = new InspectorMethod(node.type, name, className, node.start, node.end, node.loc.start.line, args, returns);
    arr.push(result);
  },

  /**
   *  Private: parses `ReturnStatement` {Node}s and adds results to `returnsArr`.
   *
   *  * `node` {Node} of type `MethodDefinition`
   *  * `arr` {Array} of {InspectorMethod} results. Not used, but left to have the
   *    same structure as the other type handler methods.
   *  * `returnsArr` {Array} of {ReturnResult}, the result of parsing each
   *    `ReturnStatement` {Node} is add to this.
   */
  ReturnStatement: function ReturnStatement(node, arr, returnsArr) {
    var name = false;
    try {
      name = node.argument.name;
    } catch (err) {/**/}
    var funcLine = false;
    var currentNode = node.parent;
    while (!funcLine) {
      var handler = _getHandler(currentNode.type, _typeHandlers);
      if (handler) {
        funcLine = currentNode.loc.start.line;
      } else {
        currentNode = currentNode.parent;
      }
    }
    var result = new ReturnResult(node.start, node.end, funcLine, name);
    returnsArr.push(result);
  }
};

/**
 *  ContentInspector Class
 */

var ContentInspector = function () {
  /**
   *  Public: creates an instance of ContentInspector to parse `content`.
   *
   *  Public: ContentInspector inpsect Spidermonkey AST {Node}s to process method
   *  attributes. When the content inspector finishes the last node it resolves the
   *  `promise` {Promise} property passing an {Array} of {InspectorMethod}s back.
   *
   *  * `content` {String} the raw content of the javascript file, used to identify the last {Node}.
   *
   *  ## Examples
   *
   *  ```js
   *  const commentParser = new CommentParser();
   *  const contentInspector = new ContentInspector(this.content);
   *  falafel(this.content, {
   *    sourceType: 'module',
   *    ecmaVersion: '6',
   *    onComment: commentParser.parseComment.bind(commentParser),
   *    locations: true,
   *    allowHashBang: true,
   *  }, (node) => {
   *    contentInspector.inspectNode(node);
   *  });
   *  ```
   */
  function ContentInspector(content) {
    var _this = this;

    _classCallCheck(this, ContentInspector);

    this.content = content;
    this.functions = [];
    this.returns = [];
    this.promise = new Promise(function (resolve) {
      _this.resolve = resolve;
    });
  }
  /**
   *  Public: gathers information about the methods in the `content` and adds results
   *  to `this.functions`.
   *
   *  * `node` A Spidermonkey AST {Node} to inspect.
   *
   *  ## Examples
   *
   *  ```js
   *  const commentParser = new CommentParser();
   *  const contentInspector = new ContentInspector(this.content);
   *  falafel(this.content, {
   *    sourceType: 'module',
   *    ecmaVersion: '6',
   *    onComment: commentParser.parseComment.bind(commentParser),
   *    locations: true,
   *    allowHashBang: true,
   *  }, (node) => {
   *    contentInspector.inspectNode(node);
   *  });
   *  ```
   */


  _createClass(ContentInspector, [{
    key: 'inspectNode',
    value: function inspectNode(node) {
      var handler = _getHandler(node.type, _typeHandlers);
      if (handler) handler(node, this.functions, this.returns);
      if (node.end === this.content.length) this.resolve(this.functions);
    }
  }]);

  return ContentInspector;
}();

exports.default = ContentInspector;
module.exports = exports['default'];